<?php namespace Myriad\Validators;

use Illuminate\Translation\Translator;
use Illuminate\Validation\Validator;

/**
 * Class CustomValidators
 *
 * @package App\Services
 */
class MyriadValidators extends Validator {

  /**
   * @var array default messages for new validators
   */
  private $myriadMessages = [
    'csl' => 'Value must be a comma separated list',
    'uuid' => 'Value must be a uuid conforming to RFC 4122',
    'locale' => 'Value must be a local conforming to RFC 5646'
  ];

  /**
   * Create a new Validator instance.
   *
   * @param  Illuminate/Translation/Translator $translator
   * @param  array $data
   * @param  array $rules
   * @param  array $messages
   * @param  array $customAttributes
   */
  public function __construct(Translator $translator, array $data, array $rules, array $messages = [], array $customAttributes = []) {
    $this->fallbackMessages();


    $messages = array_merge($this->myriadMessages, $messages);
    parent::__construct($translator, $data, $rules, $messages, $customAttributes);
  }

  /**
   * Validates a comma separated values
   *
   * @param string $attribute
   * @param string $value
   * @param mixed $parameters
   *
   * @return bool
   */
  public function validateCsv($attribute, $value, $parameters) {
    // parse the csv, if empty fail quickly
    $values = str_getcsv($value);
    if (empty($values)) {
      return false;
    }

    // check each value against provided rules
    if (!empty($parameters)) {
      $rules = preg_split('/[\\\]+/', join(',', $parameters));
      for ($i = 0; $i < count($values); $i++) {
        $value = $values[$i];
        foreach ($rules as $ruleString) {
          list($rule, $ruleParameters) = $this->parseRule($ruleString);
          $method = "validate{$rule}";
          if (!$this->$method($attribute, $value, $ruleParameters, $this)) {
            $this->addFailure($attribute, $rule, $ruleParameters);
            false;
          }
        }
      }
    }

    // if we made it to here, all is good
    return true;
  }

  /**
   * Validate a UUID
   *
   * @param $attribute
   * @param $value
   * @param $parameters
   *
   * @return bool
   */
  public function validateUuid($attribute, $value, $parameters) {
    try {
      $uuid = \Ramsey\Uuid\Uuid::fromString($value);
      if (isset($parameters[0])) {
        return $uuid->getVersion() == $parameters[0];
      }
      return true;
    } catch (\InvalidArgumentException $e) {
      return false;
    }
  }

  /**
   * Validates a comma separated list
   *
   * @param string $attribute
   * @param string $value
   * @param mixed $parameters
   *
   * @return bool
   */
  public function validateLocale($attribute, $value, $parameters) {
    $lang = '(?P<language>([a-z]{2,3}))';
    $region = '(?P<region>([a-z]{2})|([0-9]{3}))';
    $others = '((-[a-z0-9]+)*)';
    $pattern = '/^' . $lang . '-' . $region . $others . '$/i';

    return preg_match($pattern, $value, $matches) > 0;
  }

}